#include "max7219.h"
//------------------------------------------------
#define CLOCK_SPEED_HZ (10000000) // 10 MHz
#define ALL_DIGITS 8
//==================================================
void Send_7219 (max7219_t *dev, uint16_t *buf, uint8_t sz)
{
  spi_transaction_t t;
  memset(&t, 0, sizeof(t));
  for(uint8_t i=0; i<sz; i++)
  {
    t.length = 16;
    t.tx_buffer = buf + i;
    spi_device_transmit(dev->spi_dev, &t);
  }
}
//==================================================
void Number_7219 (max7219_t *dev, volatile long n)
{
  uint8_t ng=0;//  
  uint16_t buf[ALL_DIGITS] = {0};
  if(n<0)
  {
    ng=1;
    n*=-1;
  }
  uint8_t i=0;
  do
  {
    ++i;
    buf[i-1] = i | (n%10)<<8;// 
    n/=10;
  } while(n);
  if(ng)
  {
    buf[i-1] = (i+1) | 0x0A<<8;// -
  }
  Send_7219(dev, buf, ALL_DIGITS);
}
//==================================================
void Clear_7219 (max7219_t *dev)
{
  uint8_t i=ALL_DIGITS;
  uint16_t buf[ALL_DIGITS] = {0};
  do
  {
    buf[8-i] = i | 0x0F<<8;
  } while (--i);
  Send_7219(dev, buf, ALL_DIGITS);
}
//==================================================
void Init_7219 (max7219_t *dev, spi_host_device_t host, gpio_num_t cs_pin)
{
  uint16_t buf[1] = {0};
  memset(&dev->spi_cfg, 0, sizeof(dev->spi_cfg));
  dev->spi_cfg.spics_io_num = cs_pin;
  dev->spi_cfg.clock_speed_hz = CLOCK_SPEED_HZ;
  dev->spi_cfg.mode = 0;
  dev->spi_cfg.queue_size = 1;
  dev->spi_cfg.flags = SPI_DEVICE_NO_DUMMY;
  spi_bus_add_device(host, &dev->spi_cfg, &dev->spi_dev);

  buf[0] = 0x0F | 0x00<<8;//  
  Send_7219(dev, buf, 1);
  buf[0] = 0x0C | 0x00<<8;// 
  Send_7219(dev, buf, 1);
  buf[0] = 0x0B | (ALL_DIGITS-1)<<8;//-  
  Send_7219(dev, buf, 1);
  buf[0] = 0x09 | 0xFF<<8;//  
  Send_7219(dev, buf, 1);
  buf[0] = 0x0A | 0x02<<8;// 
  Send_7219(dev, buf, 1);
  buf[0] = 0x0C | 0x01<<8;// 
  Send_7219(dev, buf, 1);
  Clear_7219(dev);
}
//==================================================
